


# LLM-Experiment Series

Welcome to the LL-Experiment series! This series of notebooks and scripts aims to provide a comprehensive guide on investigating the internal workings of Large Language Models (LLMs), understanding how they process inputs, and experimenting with their architectures.

## Table of Contents

- [Introduction](#introduction)
- [Series Overview](#series-overview)
- [Getting Started](#getting-started)
- [Notebooks and Scripts](#notebooks-and-scripts)
- [Contributing](#contributing)
- [License](#license)

## Introduction

Large Language Models (LLMs) have revolutionized the field of natural language processing (NLP) by achieving state-of-the-art performance on various tasks. However, understanding their internal workings and how they process inputs can be challenging. This series aims to demystify LLMs by providing detailed explanations, hands-on experiments, and practical tips for tweaking their architectures.

## Series Overview

The LLM-Experiment series will cover the following topics:

1. **Understanding LLM Architectures**:
   - An overview of popular open source LLM architectures like Whisper, Llama, and Mixtral.
   - Key trouble shooting during experimentation

2. **Investigating Input Processing**:
   - How inputs are tokenized and embedded.
   - The role of attention mechanisms in processing sequences.
   - Visualizing and analyzing the outputs at various layers of the model.

3. **Tweaking LLM Architectures**:
   - Experimenting with different configurations and hyperparameters.
   - Modifying existing LLM architectures to improve performance or adapt to specific tasks.
   - Implementing custom layers and components.

4. **Conducting New Experiments**:
   - Designing and implementing new experiments to test hypotheses about LLM behavior.
   - Evaluating the impact of architectural changes on model performance.
   - Sharing insights and findings with the community.

## Getting Started

To get started with the LLM-Experiment series, you will need the following:

1. **Python Environment**:
   - All these notebooks are created in Kaggle or Google Colab, So it's recommended to use the same to reproduce the results for other models


2. **Hugging Face Account**:
   - Create a Hugging Face account and obtain an API token.
   - Login to Hugging Face using the provided token or username and token.
   - Most of the Mistral,Llama models needs some sort of Agreement acceptance

3. **Notebooks and Scripts**:
   - Clone this repository to access the notebooks and scripts or you can directly open in Google Colab 
   - Follow the instructions in each notebook to run the experiments and analyze the results.

## Notebooks and Scripts

The series will include the following notebooks and scripts:

1. **Mixtral Model Analysis**:
   - Analyzing the architecture and configuration of the Mixtral model.
   - Registering hooks to capture the outputs at various layers.

2. **Input Processing and Embedding**: - Upcoming


3. **Attention Mechanisms and improvements**: - Upcoming


4. **Rolling Buffer,KV-cache,Sliding Window Attention**: - Upcoming


5. **Tweaking Model Architectures - Adapters,Down-Casting**: - Upcoming
   

## Contributing

We welcome contributions from the community! If you have any ideas, suggestions, or improvements, please feel free to open an issue or submit a pull request.

## License

This project is licensed under the MIT License. See the [LICENSE](LICENSE) file for more details.


